# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['entries', 'entries.migrations', 'entries.tests']

package_data = \
{'': ['*'],
 'entries': ['docs/*',
             'static/css/*',
             'static/img/favicons/*',
             'templates/*',
             'templates/base_template/*',
             'templates/entries/*']}

install_requires = \
['Django>=4.0,<5.0',
 'Markdown>=3.3.6,<4.0.0',
 'bleach>=4.1.0,<5.0.0',
 'django-crispy-forms>=1.13.0,<2.0.0',
 'django-extensions>=3.1.5,<4.0.0',
 'markdownify>=0.10.0,<0.11.0',
 'types-Markdown>=3.3.8,<4.0.0',
 'types-bleach>=4.1.1,<5.0.0']

setup_kwargs = {
    'name': 'django-entries',
    'version': '0.1.11',
    'description': 'Entries is a helper Django app with CRUD functions based on htmx.',
    'long_description': '# django-entries\n\n## Overview\n\nBasic create-read-update-delete (CRUD) functionality for an `Entry` model.\n\nThe base [template](./entries/templates/base.html) makes use of light css and javascript:\n\n1. `starter.css` [stylesheet](./entries/static/css/starter.css)\n2. `pylon` 0.1.1 for `<hstack>` and `<vstack>` layouts\n3. `htmx` 1.6.1 for html-over-the-wire functionality, e.g. [infinite scrolling](./entries/docs/infinity_scroll.md)\n4. `hyperscript` 0.9 for client-side reactivity\n5. `simplemde` a simple markdown editor\n\n## Quickstart\n\nInstall in your virtual environment:\n\n```zsh\n.venv> pip3 install django-entries # poetry add django-entries\n```\n\nInclude package in main project settings file:\n\n```python\n# in project_folder/settings.py\nINSTALLED_APPS = [\n    ...,\n    \'crispy_forms\',  # add crispy_forms at least > v1.13, if not yet added\n    \'entries\' # this is the new django-entries folder\n]\n\n# in project_folder/urls.py\nfrom django.views.generic import TemplateView\nfrom django.urls import path, include # new\nurlpatterns = [\n    ...,\n    path(\'entry/\', include(\'entries.urls\')), # new\n    path("", TemplateView.as_view(template_name="home.html")), # (optional: if fresh project install)\n]\n```\n\nAdd to database:\n\n```zsh\n.venv> python manage.py migrate # adds the `Entry` model to the database.\n.venv> python manage.py createsuperuser # (optional: if fresh project install)\n```\n\nLogin to add:\n\n```zsh\n.venv> `python manage.py runserver`\n# Visit http://127.0.0.1:8000/entry/entries/list\n# Assumes _entry_ as folder in config/urls.py\n# The `Add entry` button is only visible to logged in users.\n# Can login via admin using the superuser account http://127.0.0.1:8000/admin/\n# Visit the list page again at http://127.0.0.1:8000/entry/entries/list to see the `Add entry` button.\n```\n\n## Test\n\n```zsh\n.venv> pytest --ds=config.settings --cov\n```\n',
    'author': 'Marcelino G. Veloso III',
    'author_email': 'mars@veloso.one',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/justmars/django-entries',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from django.contrib.auth.decorators import login_required
from django.core.paginator import EmptyPage, Paginator
from django.db.models import Q
from django.http.request import HttpRequest
from django.http.response import HttpResponse
from django.shortcuts import get_object_or_404, redirect
from django.template.response import TemplateResponse
from django.urls import reverse
from django.views.decorators.http import require_http_methods

from .forms import EntryForm
from .models import Entry

EDITOR = "entries/entry_edit.html"
DETAIL = "entries/entry_detail.html"
FIRST_PAGE = "entries/entry_list.html"
NEXT_PAGE = "entries/entry_next.html"
ITEM_IN_PAGE = "entries/entry_item.html"
MAX_ITEMS_PER_PAGE = 2


@login_required
def edit_entry(request: HttpRequest, slug: str) -> TemplateResponse:
    entry = Entry.get_for_user(slug, request.user)
    url = reverse("entries:edit_entry", kwargs={"slug": slug})
    form = EntryForm(request.POST or None, instance=entry, submit_url=url)
    if request.method == "POST":
        if form.is_valid():
            if form.has_changed():
                form.save()
        return redirect(entry.get_absolute_url())
    context = {"form": form, "edit_header": f"Update {entry.title}"}
    return TemplateResponse(request, EDITOR, context)


@login_required
def add_entry(request: HttpRequest) -> TemplateResponse:
    url = reverse("entries:add_entry")
    form = EntryForm(request.POST or None, submit_url=url)
    if request.method == "POST" and form.is_valid():
        entry = form.save(commit=False)
        entry.author = request.user
        entry.save()
        return redirect(entry.get_absolute_url())
    context = {"form": form, "edit_header": "Write An Entry"}
    return TemplateResponse(request, EDITOR, context)


@login_required
@require_http_methods(["DELETE"])
def delete_entry(request: HttpRequest, slug: str) -> HttpResponse:
    url = reverse("entries:list_entries")
    entry = Entry.get_for_user(slug, request.user)
    response = HttpResponse(headers={"HX-Redirect": url})
    entry.delete()
    return response


def view_entry(request: HttpRequest, slug: str) -> TemplateResponse:
    entry = get_object_or_404(Entry, slug=slug)
    context = {"entry": entry, "metadata": entry.meta}
    return TemplateResponse(request, DETAIL, context)


def set_context(request: HttpRequest, loader: str) -> TemplateResponse:
    """Add to the `url` from the `request` object's `q` and `next` parameters, if present."""

    # initialize
    context = {}
    context["template_for_item"] = ITEM_IN_PAGE  # template for each page item
    url = reverse("entries:scroll_entries")  # base url for infinity scrolling
    query = request.GET.get("q", None)  # get query from url, else None
    page_number = request.GET.get("next", 1)  # get next from url, else page 1

    # adds to base and filters queryset, if query available; else default
    qs = Entry.objects.all()
    if query:
        url = f"{url}?q={query}"
        qs = qs.filter(
            Q(title__icontains=query)
            | Q(excerpt__icontains=query)
            | Q(content__icontains=query)
        )

    # set page from request
    paginator = Paginator(qs, MAX_ITEMS_PER_PAGE)
    page_obj = paginator.get_page(page_number)
    context["page_obj"] = page_obj

    # adds to base, if next page available
    next_num = None
    if page_obj.has_next:
        try:
            next_num = page_obj.next_page_number()
        except EmptyPage:
            next_num = None
    if next_num:
        context["next_page_url"] = f"{url}?next={next_num}"

    # return html fragment to the template loader
    return TemplateResponse(request, loader, context)


def list_entries(request: HttpRequest) -> TemplateResponse:
    return set_context(request, FIRST_PAGE)


def scroll_entries(request: HttpRequest) -> TemplateResponse:
    return set_context(request, NEXT_PAGE)


def view_about(request: HttpRequest) -> TemplateResponse:
    return TemplateResponse(request, "about.html", {})
